/*
 * at89-prog - at89s8253.c
 *
 * 	AT89S8253 support
 *
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <linux/ppdev.h>
#include <errno.h>
#include <at89-prog.h>

#include <spi.h>

#define PROG_ENABLE		0xac
#define PROG_ENABLE2		0x53

#define CHIP_ERASE		0xac
#define CHIP_ERASE2		0x80

#define WRITE_PAGE		0x50
#define READ_PAGE		0x30

int prog_enable(void)
{
	unsigned int i;
	uint8_t x[4];

	for (i = 0; i < 20; i++) {
		x[0] = spi_xmit(PROG_ENABLE);
		x[1] = spi_xmit(PROG_ENABLE2);
		x[2] = spi_xmit(0x00);
		x[3] = spi_xmit(0x00);
		if (x[1] == PROG_ENABLE && x[2] == PROG_ENABLE2)
			return 0;
	}
	return -ETIMEDOUT;
}

void chip_erase(void)
{
	spi_send(CHIP_ERASE);
	spi_send(CHIP_ERASE2);
	spi_send(0x00);
	spi_send(0x00);
}

int write_page(unsigned int page, const void *buf, unsigned int page_size)
{
	const uint8_t *data = buf;
	unsigned int i;
	
	spi_send(WRITE_PAGE);
	spi_send(page);

	for (i = 0; i < page_size; i++)
		spi_send(data[i]);
	return 0;
}

int read_page(unsigned int page, void *buf, unsigned int page_size)
{
	uint8_t *data = buf;
	unsigned int i;

	spi_send(READ_PAGE);
	spi_send(page);

	for (i = 0; i < page_size; i++)
		data[i] = spi_recv();
	return 0;
}

int read_signature(u8 *signature, unsigned int len)
{
	unsigned int i;

	if (len > 32)
		len = 32;
 
	for (i = 0; i < len; i++) {
		spi_xmit(0x28);
		spi_xmit(i);
		spi_xmit(0);
		signature[i] = spi_xmit(0x00);
	}
	return len;
}
